
from sqlalchemy import Column, Date, DateTime, ForeignKey, Integer, String
import sqlalchemy
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship, backref


MythosBase = declarative_base()

class Abstract():
    @classmethod
    def get_description_entries(cls, source):
        raise NotImplementedError('Must be implemented by subclass')
    
    @classmethod
    def get_list_entries(cls, source):
        raise NotImplementedError('Must be implemented by subclass')
    
    @classmethod
    def show_description(cls, source):
        entries = cls.get_description_entries(source)
        padding = max([len(key) for key, _value in entries])
        for key, value in entries:
            print "%s: %s" % (key.rjust(padding), value)
        print
    
    @classmethod
    def show_list(cls, sources):
        entries_list = [[str(value) for _key, value in cls.get_list_entries(source)] for source in sources]
        entries_list.sort()
        for entries in entries_list:
            print ", ".join(entries)
        print

class Channel(MythosBase, Abstract):
    __tablename__ = 'channel'
    
    id = Column('chanid', Integer, primary_key=True)
    number = Column('channum', String)
    call_sign = Column('callsign', String)
    name = Column(String)
    icon = Column(String)
    last_record = Column(DateTime)

    @classmethod
    def get_description_entries(cls, source):
        return [
            ("Call Sign", source.call_sign),
            ("Name", source.name),
            ("Number:", source.number),
            ("Last Record Date: ", source.last_record)
        ]
    
    @classmethod
    def get_list_entries(cls, source):
        return [
            ("Number", source.number.rjust(3)),
            ("Name", source.name)
        ]

class File(Abstract):
    path = None
    recording = None
    storage_group = None
    
    def __init__(self, path, recording, storage_group):
        self.path = path
        self.recording = recording
        self.storage_group = storage_group
        
    @classmethod
    def get_description_entries(cls, source):
        path = source.path
        if not path and source.recording:
            path = source.recording.file_name
        return [
            ("Path", path),
            ("Program ID", source.recording.program_id if source.recording else "<none>")
            ("Storage Group", source.storage_group.name if source.storage_group else "<none>"),
        ]
    
    @classmethod
    def get_list_entries(cls, source):
        return cls.get_description_entries(source)

class Recording(MythosBase, Abstract):
    __tablename__ = 'recorded'
    
    channel_id = Column('chanid', Integer, ForeignKey('channel.chanid'), primary_key=True)
    start_time = Column('starttime', DateTime, primary_key=True)
    title = Column(String)
    subtitle = Column(String)
    description = Column(String)
    season = Column(Integer)
    episode = Column(Integer)
    category = Column(String)
    original_air_date = Column('originalairdate', Date)
    file_name = Column('basename', String)
    delete_pending = Column('deletepending', Integer)
    storage_group = Column('storagegroup', String)
    program_id = Column('programid', String)
    
    channel = relationship("Channel", backref=backref('recorded'))

    @classmethod
    def get_description_entries(cls, source):
        return [
            ("Title", source.title),
            ("Subtitle", source.subtitle),
            ("Description", source.description),
            ("Channel", source.channel.call_sign),
            ("Recording Date", source.start_time),
            ("Original Air Date", source.original_air_date),
            ("File Name", source.file_name),
            ("Storage Group", source.storage_group),
            ("Program ID", source.program_id),
            ("Delete Pending", source.delete_pending)
        ]
    
    @classmethod
    def get_list_entries(cls, source):
        entries = [("Title", source.title)]
        entries.append(('Subtitle', source.subtitle if source.subtitle else '<no subtitle>'))
        entries.append(('Original Air Date', str(source.original_air_date) if source.original_air_date else '<no original air date>'))
        entries.append(('Program ID', source.program_id))
        return entries
        
class StorageGroup(MythosBase, Abstract):
    __tablename__ = 'storagegroup'
    
    id = Column('id', Integer, primary_key=True)
    group_name = Column('groupname', String)
    host_name = Column('hostname', String)
    directory_name = Column('dirname', String)

    @classmethod
    def get_description_entries(cls, source):
        return [
            ("Identifier", source.id),
            ("Group Name", source.group_name),
            ("Host Name", source.host_name),
            ("Directory Name", source.directory_name),
        ]
    
    @classmethod
    def get_list_entries(cls, source):
        return cls.get_description_entries(source)
        
   